//------------------------------------------------------------------------------
//	$Id: glasses.js,v 1.2 2023/08/06 01:06:37 wolf Exp $
//------------------------------------------------------------------------------
//	Lizenz: WTF public license (see: http://www.wtfpl.net/)
//------------------------------------------------------------------------------

/*jsl:option explicit*/

'use strict';

(function(){

	const MAIN_CLASS = 'glasses';

	//----------------------------------------------------------------------
	//	update position values from event
	//----------------------------------------------------------------------

	function updatePositionFromEvent( svgElement, evt ){

		if( evt.touches ){ evt = evt.touches[ 0 ]; }

		const pt = svgElement.createSVGPoint();

		pt.x = evt.clientX;
		pt.y = evt.clientY;

		const pos = pt.matrixTransform(
					svgElement.getScreenCTM().inverse() );

		svgElement.dataset.posX = pos.x;
		svgElement.dataset.posY = pos.y;
	}

	//----------------------------------------------------------------------
	//	move glasses and clippings by setting transform
	//----------------------------------------------------------------------

	function moveGlasses( svgElement ){

		const x = svgElement.dataset.posX;
		const y = svgElement.dataset.posY;

		const transformation = 'translate(' + x + ',' + y + ')';

		const elements =
			svgElement.getElementsByClassName( 'glasses-dynamic' );

		for( var e=0 ; e<elements.length; ++e ){

			const element = elements[ e ];
			element.setAttribute( 'transform', transformation );
		}
	}

	//----------------------------------------------------------------------
	//	drag by mouse
	//----------------------------------------------------------------------

	function startDragging( svgElement, element, evt ){

		//----------------------------------------------------------------------
		//	immediately update position
		//----------------------------------------------------------------------
		//	TODO: use grab offset
		//----------------------------------------------------------------------

		updatePositionFromEvent( svgElement, evt );
		moveGlasses( svgElement );

		//----------------------------------------------------------------------
		//	mouse event handlers
		//----------------------------------------------------------------------

		function processFinish( evt ){

			svgElement.removeEventListener( 'mousemove' , processMove  , false);
			svgElement.removeEventListener( 'mouseup'   , processFinish, false);

			element.style.cursor = 'grab';
		}

		function processMove( evt ){

			if( !evt.buttons ){ processFinish( evt ); return; }

			updatePositionFromEvent( svgElement, evt );
			moveGlasses( svgElement );

			element.style.cursor = 'crosshair';
		}

		svgElement.addEventListener( 'mousemove' , processMove  , false);
		svgElement.addEventListener( 'mouseup'   , processFinish, false);
	}

	//----------------------------------------------------------------------
	//	touch handler
	//----------------------------------------------------------------------

	function processTouch( evt ){

		if( evt.preventDefault){ evt.preventDefault(); }

		updatePositionFromEvent( this, evt );
		moveGlasses( this );
	}

	//----------------------------------------------------------------------
	//	attach handler to selected svgElement
	//----------------------------------------------------------------------

	function attachHandler( svgElement ){

		//--------------------------------------------------------------
		//	SVG dom object width
		//--------------------------------------------------------------

		const viewBox = svgElement.getAttribute('viewBox').split(/\s+/);
		const viewWidth = parseFloat( viewBox[2] );

		svgElement.style.width = viewWidth + 'px';

		//--------------------------------------------------------------
		//	attach mouse down handler
		//--------------------------------------------------------------

		const movableElement =
			svgElement.getElementsByClassName( 'glasses' )[0];

		movableElement.addEventListener( 'mousedown',  function( evt ){

			evt.stopPropagation( true );
			startDragging( svgElement, this, evt );
		});

		movableElement.style.cursor = 'grab';

		//--------------------------------------------------------------
		//	attach touch handler
		//--------------------------------------------------------------

		svgElement.addEventListener( 'touchstart', processTouch, false );
		svgElement.addEventListener( 'touchmove',  processTouch, false );

		//--------------------------------------------------------------
		//	initial move
		//--------------------------------------------------------------

		moveGlasses( svgElement );
	}

	//----------------------------------------------------------------------
	//	after page load add logic to selected SVG elements
	//----------------------------------------------------------------------

	window.addEventListener( 'load', function(){

		const svgElements = document.getElementsByTagName( 'svg' );
		for( var s = 0; s < svgElements.length; ++s ){

			const svgElement = svgElements[s];

			if( svgElement.classList.contains( MAIN_CLASS ) ){

				attachHandler( svgElement );
			}
		}
	});
})();

//------------------------------------------------------------------------------
//	$Id: glasses.js,v 1.2 2023/08/06 01:06:37 wolf Exp $
//------------------------------------------------------------------------------
